#if !defined SOBEL
#define SOBEL

#include "cv.h"

class Sobel {

  private:

	  // private attributes
	  IplImage *output;

  public:

	  // empty constructor
	  Sobel() : output(0) { } 

	  IplImage* getOutputImage() {

		  return output;
	  }

	  // to check if an initialization is required
	  bool isInitialized(IplImage *image) {

		  return output && (output->width == image->width) 
			            && (output->height == image->height);
	  }

	  // for all memory allocation
	  void initialize(IplImage *image) {

		  cvReleaseImage(&output);
		  output= cvCreateImage(cvSize(image->width,image->height),
										image->depth, image->nChannels);
	  }

	  // the processing of the image
	  void process(IplImage *image);

	  // the method that checks for initilization
	  // and then process the image
	  inline void processImage(IplImage *image) {

		  if (!isInitialized(image)) {

			  initialize(image);
		  }

		  process(image);
	  }

	  // memory de-allocation
	  void release() {

		  cvReleaseImage(&output);
	  }

	  ~Sobel() {

		  release();
	  }
};

#endif
